/*
 * File: receiver.c
 * Author: Russell J Clark
 * Date: October 1, 1996
 *
 * This module implements the receiver code.  The receiver reads the
 * data then compares it to the actual data sent to determine if an
 * error has occurred. It also calculates the checksum over the received
 * data to determine if the checksum correctly identified whether
 * errors were present.
 *
 */

#include <fcntl.h>
#include <stdio.h>
#include <sys/types.h>

#include "nwsim.h"

main(argc,argv)
int argc;
char *argv[];
{

    unsigned char  compareData[PACKET_TOTAL_SIZE];
    unsigned char  messageData[PACKET_TOTAL_SIZE];
    unsigned char  errorCheckData[PACKET_TRAILER_SIZE];
    int 	datafile;
    int bytesRead, bytesReceived;

    if (argc < 2)
    {
        fprintf(stderr,"Usage: %s data_file \n",argv[0]);
        exit(1);
    }



    /*
     * Open the data file for reading the data for comparison.
     */
 
    if  ((datafile = open(argv[1],O_RDONLY)) < 0 ) {
        printf("%s: failed to open data file %s\n",argv[0],argv[1]);
        exit();
    }
 
    if  ((bytesRead = read(datafile,compareData,PACKET_DATA_SIZE)) <
            PACKET_DATA_SIZE )
    {
        printf("%s: failed to read %d bytes from file %s\n",argv[0],
                PACKET_DATA_SIZE,argv[1]);
        printf("%s: only %d bytes read \n",argv[0],bytesRead);
        exit();
    } 

    if ((bytesReceived = fread(messageData,1,PACKET_TOTAL_SIZE,stdin))
		< PACKET_TOTAL_SIZE) 
    {
       	printf("%s: recvfrom failed to read %d bytes\n",argv[0],
                PACKET_TOTAL_SIZE);
       	printf("%s: only %d bytes read \n",argv[0],bytesReceived);
    } else {
	printf("%s: message of size %d bytes received\n",argv[0],bytesReceived);
	/*
	 * Calculate the error detection code.
	 */
	errorCheckData[0] = CalculateEvenParity(messageData,PACKET_DATA_SIZE);

	printf("%s: Received Parity is %d\n",argv[0],messageData[PACKET_DATA_SIZE]);
	printf("%s: Calculated Parity is %d\n",argv[0],errorCheckData[0]);

	/*
	 * See if the calculated parity is the same as the one sent.
	 */

	if (memcmp(errorCheckData,&messageData[PACKET_DATA_SIZE],
			PACKET_TRAILER_SIZE) == 0)
		
	   printf("%s: The parity bit indicates data is correct\n",argv[0]);
	else
	   printf("%s: The parity bit indicates data is NOT correct\n",argv[0]);

	/*
	 * Now compare the message to the real data and see if it is right.
	 */

	if (memcmp(compareData,messageData,PACKET_DATA_SIZE) == 0)
	    printf("%s: In reality, data is correct\n",argv[0]);
	else
	    printf("%s: In reality, data is NOT correct\n",argv[0]);

    } /* else */

} /* main */
